﻿using System;
using System.ComponentModel;
using System.Globalization;
using System.Resources;
using System.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebPartPages;

namespace RH.DiscussionListUpload
{
    public class DiscussionListUpload : WebPart
    {
        private string _PicLib = default(string);
        private string _DocLib = default(string);
        private string _Message = string.Empty;
        public string ErrorMessage = string.Empty;
        private HtmlTable _Table;
        private FileUpload _Image;
        private FileUpload _File;
        private Button _ButtonImage;
        private Button _ButtonFile;

        private CultureInfo _Cult;
        private ResourceManager _RM;

        #region Properties
        [Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Shared),
        FriendlyName("Text"), Description("Picture Library")]
        public string PicLib
        {
            get { return _PicLib; }
            set { _PicLib = value; }
        }

        [Browsable(false), Category("Miscellaneous"), DefaultValue(""), WebPartStorage(Storage.Shared),
        FriendlyName("Text"), Description("Document Library")]
        public string DocLib
        {
            get { return _DocLib; }
            set { _DocLib = value; }
        }
        #endregion

        #region overrides
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            try
            {
                _RM = new ResourceManager("RH.DiscussionListUpload.Localization.DiscussionListUpload", GetType().Assembly);
                _Cult = GetCultureInfo();

                _Table = new HtmlTable { ID = "DiscussionListUploadWebpart"};
                Controls.Add(_Table);
            }
            catch (Exception ex)
            {
                ErrorMessage += string.Format("{0} (OnInit):\r\n{1}", _RM.GetString("Error", _Cult), ex);
            }
        }

        protected override void CreateChildControls()
        {
            try
            {
                if (!string.IsNullOrEmpty(ErrorMessage)) return;

                // check if there is a document library and a picture library
                GetlibraryNames(SPContext.Current.Web);

                if (!string.IsNullOrEmpty(_PicLib))
                {
                    var row = new HtmlTableRow();
                    var cellText = new HtmlTableCell { InnerText = string.Format("{0} ", _RM.GetString("UploadImage", _Cult)) };
                    row.Controls.Add(cellText);

                    var cell = new HtmlTableCell();
                    _Image = new FileUpload();
                    cell.Controls.Add(_Image);
                    row.Controls.Add(cell);

                    cell = new HtmlTableCell();
                    _ButtonImage = new Button { Text = _RM.GetString("Upload", _Cult), CommandName = "Type", CommandArgument = "Image" };
                    _ButtonImage.Click += _button_ServerClick;
                    cell.Controls.Add(_ButtonImage);
                    row.Controls.Add(cell);

                    _Table.Controls.Add(row);
                }
                else
                {
                    _Message += string.Format("<div style=\"padding: 20px; margin: 20px; border-style: groove; position: absolute; top: 50%; left: 45%; background-color: #FFFFFF;\" align=\"center\">" +
                                              "{0}<strong>{1}</strong></div>", _Message, _RM.GetString("CreatePictureLibrary", _Cult));
                }

                if (!string.IsNullOrEmpty(_DocLib))
                {
                    var row = new HtmlTableRow();
                    var cellText = new HtmlTableCell { InnerText = string.Format("{0} ", _RM.GetString("UploadFile", _Cult)) };
                    row.Controls.Add(cellText);

                    var cell = new HtmlTableCell();
                    _File = new FileUpload();
                    cell.Controls.Add(_File);
                    row.Controls.Add(cell);

                    cell = new HtmlTableCell();
                    _ButtonFile = new Button { Text = _RM.GetString("Upload", _Cult), CommandName = "Type", CommandArgument = "File" };
                    _ButtonFile.Click += _button_ServerClick;
                    cell.Controls.Add(_ButtonFile);
                    row.Controls.Add(cell);

                    _Table.Controls.Add(row);
                }
            }
            catch (Exception ex)
            {
                ErrorMessage += string.Format("{0} (CreateChildControls):\r\n{1}", _RM.GetString("Error", _Cult), ex);
            }
        }

        protected override void RenderContents(HtmlTextWriter writer)
        {
            try
            {
                if (String.IsNullOrEmpty(ErrorMessage))
                {
                    //EnsureChildControls();
                    base.RenderContents(writer);

                    //_Table.RenderControl(writer);
                    writer.Write(_Message);
                }
                else
                {
                    var errorControl = new HtmlGenericControl("DIV");
                    errorControl.Attributes.Add("style", "padding: 20px; margin: 20px; border-style: groove; background-color: #FFFFFF;");
                    errorControl.InnerHtml = "<IMG src=\"/_layouts/images/error16by16.gif\" border=\"0\"/>" + ErrorMessage;
                    errorControl.RenderControl(writer);
                }
            }
            catch (SecurityException ex)
            {
                ErrorMessage = string.Format("Please set the trust level in your web.config to 'WSS_Medium' or 'Full'<br/>{0}<br/>", ex);
            }
            catch (Exception ex)
            {
                var errorControl = new HtmlGenericControl("DIV");
                errorControl.Attributes.Add("style", "padding: 20px; margin: 20px; border-style: groove; background-color: #FFFFFF;");
                errorControl.InnerHtml = "<IMG src=\"/_layouts/images/error16by16.gif\" border=\"0\"/>" + ex;
                writer.Write(ex.ToString());
            }
        }
        #endregion

        void _button_ServerClick(object sender, EventArgs e)
        {
            try
            {
                SPWeb web = SPContext.Current.Web;
                if (_Image != null && _Image.FileBytes.Length != 0)
                {
                    UploadFile(web, _PicLib, _Image.FileName, _Image.FileBytes);
                    InsertLinkIntoDiscussion(web, _Image.FileName, FileType.Image);
                }
                else if (_File != null && _File.FileBytes.Length != 0)
                {
                    UploadFile(web, _DocLib, _File.FileName, _File.FileBytes);
                    InsertLinkIntoDiscussion(web, _File.FileName, FileType.File);
                }
            }
            catch (Exception ex)
            {
                ErrorMessage += string.Format("{0} (_button_ServerClick):\r\n{1}", _RM.GetString("Error", _Cult), ex);
            }
        }

        private void UploadFile(SPWeb web, string listName, string fileName, byte[] bytes)
        {
            try
            {
                SPList list = Helper.GetList(web, listName);
                if (list == null)
                {
                    _Message += string.Format(_RM.GetString("ListNotFound", _Cult), listName);
                    return;
                }
                SPFolder folder = list.RootFolder;
                SPFileCollection files = folder.Files;

                SPFile file;
                try
                {
                    string FileUrl = web.Url + "/" + folder.Url + "/" + fileName;
                    file = web.GetFile(FileUrl);
                }
                catch (ArgumentException)
                {
                    return;
                }

                if (!file.Exists)
                {
                    var fileNameWithPath = string.Format("{0}/{1}", folder.Url, fileName);
                    files.Add(fileNameWithPath, bytes);
                    _Message = string.Format("{0} ", _RM.GetString("UploadCompleted", _Cult));
                }
                else
                {
                    _Message = string.Format(_RM.GetString("FileExists", _Cult) + " ", fileName);
                }
            }
            catch (Exception ex)
            {
                _Message += string.Format("{0} (UploadFile):\r\n{1}", _RM.GetString("Error", _Cult), ex);
            }
        }

        /// <summary>
        /// if not set, will set the PicLib/DocLib with the first list we found
        /// </summary>
        /// <param name="web"></param>
        private void GetlibraryNames(SPWeb web)
        {
            // Write Doclib/Piclib Titles in Webpart Property, if not yet set
            if (!string.IsNullOrEmpty(_PicLib) || !string.IsNullOrEmpty(_DocLib)) return;

            foreach (SPList list in web.Lists)
            {
                if (list.BaseTemplate == SPListTemplateType.PictureLibrary)
                    _PicLib = list.Title;

                if (list.BaseTemplate == SPListTemplateType.DocumentLibrary && list.ID != SPContext.Current.ListId)
                    _DocLib = list.Title;
            }
        }

        /// <summary>
        /// update Discussion entry with url to file/image
        /// </summary>
        /// <param name="web"></param>
        /// <param name="filename"></param>
        /// <param name="fileType"></param>
        private void InsertLinkIntoDiscussion(SPWeb web, string filename, FileType fileType)
        {
            try
            {
                SPList discussionList = SPContext.Current.List;
                SPList library = fileType == FileType.Image ? web.Lists[_PicLib] : web.Lists[_DocLib];

                if (!ContainsField(discussionList, SPBuiltInFieldId.Body)) return;
                string bodyFieldName = discussionList.Fields[SPBuiltInFieldId.Body].Title;

                // create text to insert
                string path = string.Format("{0}/{1}/{2}", web.Url, library.RootFolder.Url, filename);
                string urlToInsert = (fileType == FileType.Image) ?
                    string.Format("<img src=\"{0}\" border=\"0\" alt=\"{1}\">", path, filename) :
                    string.Format("<a href=\"{0}\">{1}</a>", path, filename);

                // create javascript
                // if the existing text contains a <HR>, it is most likely a reply. so add the picture at the beginning instead of the end
                string js = "\r\n";
                //js += "debugger;";
                js += Helper.JavaScriptGetTagFromIdentifierAndTitle();
                js += "var discussionBody = getTagFromIdentifierAndTitle('textarea','TextField','" + bodyFieldName + "');\r\n";
                js += "if (discussionBody.value.length > 6)\r\n";
                js += "{\r\n";
                js += "     if (discussionBody.value.contains('<HR>'))\r\n";
                js += "     {\r\n";
                js += "         discussionBody.value = discussionBody.value.substr(5);\r\n";
                js += "         discussionBody.value = '<DIV>' + '"+urlToInsert+"' + discussionBody.value;\r\n";
                js += "     }\r\n";
                js += "     else\r\n";
                js += "     {\r\n";
                js += "         discussionBody.value = discussionBody.value.substr(0, discussionBody.value.length-6) + '" + urlToInsert + "' + '</DIV>';\r\n";
                js += "     }\r\n";
                js += "}\r\n";
                js += "else if (discussionBody.value.length == 0)\r\n";
                js += "{\r\n";
                js += "     discussionBody.value = '<DIV>' + '" + urlToInsert + "' + '</DIV>';\r\n";
                js += "}\r\n";

                // add javascript to page
                ScriptManager.RegisterStartupScript(this, typeof(DiscussionListUpload), "setValue", js, true);
            }
            catch (Exception ex)
            {
                _Message += string.Format("{0} (InsertLinkIntoDiscussion):\r\n{1}", _RM.GetString("Error", _Cult), ex.Message);
            }
        }

        public string GetDesignTimeHtml()
        {
            return SPEncode.HtmlEncode(_RM.GetString("DesignTimeHTML", _Cult));
        }

        /// <summary>
        /// returns the current cultureinfo
        /// </summary>
        /// <returns></returns>
        private static CultureInfo GetCultureInfo()
        {
            CultureInfo newCultureInfo;
            try
            {
                uint localeID = SPContext.Current.RegionalSettings.LocaleId;
                newCultureInfo = new CultureInfo((int)localeID);
            }
            catch
            {
                newCultureInfo = new CultureInfo(1033);
            }

            return newCultureInfo;
        }

        /// <summary>
        /// Does a list contain a field with this Guid?
        /// </summary>
        /// <param name="list"></param>
        /// <param name="guid"></param>
        /// <returns></returns>
        private bool ContainsField(SPList list, Guid guid)
        {
            try
            {
                foreach (SPField field in list.Fields)
                {
                    if (field.Id == guid)
                        return true;
                }
            }
            catch (Exception ex)
            {
                ErrorMessage += string.Format("{0} (ContainsField):\r\n{1}", _RM.GetString("Error", _Cult), ex.Message);
            }
            return false;
        }

        public override ToolPart[] GetToolParts()
        {
            try
            {
                var toolparts = new ToolPart[3];
                var wptp = new WebPartToolPart();
                var custom = new CustomPropertyToolPart();
                var footer = new CustomPropertyToolPart();
                toolparts[0] = custom;
                toolparts[1] = wptp;
                toolparts[2] = footer;

                // Theese are the custom ToolParts
                toolparts[0] = new CustomToolPart();
                toolparts[2] = new CustomToolPartFooter();

                return toolparts;

            }
            catch (Exception ex)
            {
                string error = string.Format("Error in GetToolParts: {0}", ex);
                Helper.WriteULSError(error);
                ErrorMessage += error;
            }
            return null;
        }
    }
}
