﻿using System;
using Microsoft.SharePoint;

namespace RH.DiscussionListUpload
{
    class Helper
    {
        internal static SPList GetList(SPWeb web, string listName)
        {
            foreach (SPList list in web.Lists)
            {
                if (list.Title == listName)
                    return list;
            }

            return null;
        }

        /// <summary>
        /// Check if the list exists in the web
        /// </summary>
        /// <param name="web"></param>
        /// <param name="listId">Guid</param>
        /// <returns></returns>
        internal static SPList GetList(SPWeb web, Guid listId)
        {
            try
            {
                SPList list = web.Lists.GetList(listId, true);
                return list;
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error in 'GetList' for Web '{0}' and ListId '{1}': {2}", web.Url, listId, ex));
                return null;
            }
        }

        internal static SPEventReceiverDefinition GetEventReceiver(SPEventReceiverDefinitionCollection receivers, string receiverName)
        {
            foreach (SPEventReceiverDefinition receiver in receivers)
            {
                if (receiver.Class == receiverName)
                {
                    return receiver;
                }
            }
            return null;
        }

        /// <summary>
        /// write to the ULS Log (TraceSeverity = Exception)
        /// </summary>
        /// <param name="text"></param>
        internal static void WriteULSError(string text)
        {
            try
            {
                TraceProvider.RegisterTraceProvider();
                TraceProvider.WriteTrace(0, TraceProvider.TraceSeverity.Exception, Guid.Empty, "RH.DiscussionListUpload.dll", "RH.DiscussionListUpload", "General", text);
                TraceProvider.UnregisterTraceProvider();
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// write to the ULS Log (TraceSeverity = Verbose)
        /// </summary>
        /// <param name="text"></param>
        internal static void WriteULSVerbose(string text)
        {
            try
            {
                TraceProvider.RegisterTraceProvider();
                TraceProvider.WriteTrace(0, TraceProvider.TraceSeverity.Verbose, Guid.Empty, "RH.DiscussionListUpload.dll", "RH.DiscussionListUpload", "General", text);
                TraceProvider.UnregisterTraceProvider();
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// get javascript "getTagFromIdentifierAndTitle"
        /// </summary>
        /// <returns></returns>
        internal static string JavaScriptGetTagFromIdentifierAndTitle()
        {
            string js = "function getTagFromIdentifierAndTitle(tagName, identifier, title)\r\n";
            js += "{\r\n";
            js += "    var len = identifier.length;\r\n";
            js += "    var tags = document.getElementsByTagName(tagName);\r\n";
            js += "    for (var i=0; i < tags.length; i++)\r\n";
            js += "    {\r\n";
            js += "        var tempString = tags[i].id;\r\n";
            js += "        if (tags[i].title == title && (identifier == \"\" || tempString.indexOf(identifier) == tempString.length - len))\r\n";
            js += "        {\r\n";
            js += "            return tags[i];\r\n";
            js += "        }\r\n";
            js += "    }\r\n";
            js += "    return null;\r\n";
            js += "}\r\n";

            js += "String.prototype.contains = function(t) { return this.indexOf(t) >= 0 ? true : false }\r\n";

            return js;
        }
    }
}
