﻿using System;
using System.IO;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebControls;
using System.Web.UI.WebControls;
using Microsoft.SharePoint.WebPartPages;
using RH.Update;
using WebPart=Microsoft.SharePoint.WebPartPages.WebPart;
using System.Threading;

namespace RH.DiscussionListUpload
{
    public class RHDiscussionListUploadSettingsPage : LayoutsPageBase
    {
        private readonly Version _Version = new Version(1, 1, 0, 0);
        protected Label LabelErrorMessage;
        protected Button BtnSave;
        protected CheckBox EnableUploadWebpart;
        protected InputFormSection configureUploadWebpart;
        protected Label CurrentVersion;
        protected Label LatestVersion;
        protected Button CheckLatestVersion;
        protected SPList List;
        private const string _UploadWebpartTitle = "Discussion List Upload Webpart";

        protected void Page_Load(Object sender, EventArgs e)
        {
            string[] query = Request.QueryString.GetValues("list");
            #region verify query string
            if (query == null || query.Length < 1)
            {
                ShowError("Please call this page from the Settings Menu within a list.");
                configureUploadWebpart.Visible = false;
                return;
            }
            #endregion

            try
            {
                List = Helper.GetList(SPContext.Current.Web, new Guid(query[0]));

                if (List != null)
                {
                    #region check if user has rights
                    if (!List.DoesUserHavePermissions(SPBasePermissions.ManageLists))
                    {
                        ShowError("You do not have the right to manage lists.");
                        configureUploadWebpart.Visible = false;
                        return;
                    }
                    #endregion

                    if (!Page.IsPostBack)
                    {
                        SPWeb web = SPContext.Current.Web;
                        string formUrl = List.Forms[PAGETYPE.PAGE_NEWFORM].Url;
                        SPLimitedWebPartManager webparts = web.GetLimitedWebPartManager(formUrl, PersonalizationScope.Shared);
                        foreach (WebPart webpart in webparts.WebParts)
                        {
                            if (webpart.Title != _UploadWebpartTitle) continue;

                            EnableUploadWebpart.Checked = true;
                            break;
                        }

                        CurrentVersion.Text = string.Format("Current Version: {0}<br/>", _Version);
                        LatestVersion.Text = "Latest Version:<br/>";
                    }
                }
            }
            catch (Exception ex)
            {
                throw new SPException(ex.ToString());
            }
        }

        protected void CheckLatestVersion_Click(object sender, EventArgs e)
        {
            try
            {
                var latest = new Updates("RH.DiscussionListUpload");
                LatestVersion latestVersion = latest.GetLatestVersion();
                if (latestVersion != null && latestVersion.Version != null)
                {
                    LatestVersion.Text = string.Format("Latest Version: {0}<br/>", latestVersion.Version);
                }

            }
            catch (Exception ex)
            {
                throw new SPException(ex.Message);
            }
        }

        public void BtnSave_Click(Object sender, EventArgs e)
        {
            try
            {
                SPWeb web = SPContext.Current.Web;
                WebPart uploadWebpart = new DiscussionListUpload { Title = _UploadWebpartTitle };
                string newFormUrl = List.Forms[PAGETYPE.PAGE_NEWFORM].Url;
                string editFormUrl = List.Forms[PAGETYPE.PAGE_EDITFORM].Url;

                AddRemoveWebpart(web, newFormUrl, uploadWebpart, EnableUploadWebpart.Checked);
                AddRemoveWebpart(web, editFormUrl, uploadWebpart, EnableUploadWebpart.Checked);

                web.Update();

                // redirect
                Redirect();
            }
            catch (ThreadAbortException){}
            catch (Exception ex)
            {
                throw new SPException(ex.ToString());
            }
        }

        private static void AddRemoveWebpart(SPWeb web, string formUrl, WebPart uploadWebpart, bool addWebpart)
        {
            bool alreadyAdded = false;
            SPLimitedWebPartManager webparts = web.GetLimitedWebPartManager(formUrl, PersonalizationScope.Shared);
            foreach (WebPart webpart in webparts.WebParts)
            {
                if (webpart.Title != _UploadWebpartTitle) continue;

                alreadyAdded = true;
                // set webpart from the webpart collection, to be able to delete it
                uploadWebpart = webpart;
                break;
            }

            // add the webpart
            if (!alreadyAdded && addWebpart)
            {
                Helper.WriteULSVerbose("Adding DiscussionListUpload Webpart...");
                webparts.AddWebPart(uploadWebpart, "Middle", 1);
            }
            
            // remove the webpart
            if (alreadyAdded && !addWebpart)
            {
                Helper.WriteULSVerbose("Removing DiscussionListUpload Webpart...");
                webparts.DeleteWebPart(uploadWebpart);
                SPFile file = web.GetFile(formUrl);
                string reghost = ReGhost(file);
                Helper.WriteULSVerbose(string.Format("Reghosting: {0}", reghost));
            }
        }

        /// <summary>
        /// shows an error and hides people fields an submit button
        /// </summary>
        /// <param name="errorMessage"></param>
        internal void ShowError(string errorMessage)
        {
            LabelErrorMessage.Text = string.Format("<strong>{0}</strong><br/><br/>", errorMessage);
            EnableUploadWebpart.Visible = false;
            BtnSave.Visible = false;
        }

        private void Redirect()
        {
            if (List != null)
            {
                var url = string.Format("{0}/_layouts/listedit.aspx?List={1}", SPContext.Current.Web.Url, List.ID);
                Response.Redirect(url);
            }
            else
            {
                Response.Redirect(SPContext.Current.Web.Url);
            }
        }

        static string ReGhost(SPFile file)
        {
            string output = string.Empty;
            if (file == null)
            {
                output += "File not found.";
                return output;
            }

            if (file.CustomizedPageStatus != SPCustomizedPageStatus.Customized)
            {
                output += file.ServerRelativeUrl + " was not unghosted.";
                return output;
            }

            if (file.CustomizedPageStatus == SPCustomizedPageStatus.Customized)
            {
                if (!string.IsNullOrEmpty((string)file.Properties["vti_setuppath"]))
                {
                    file.Properties["vti_hasdefaultcontent"] = "false";

                    var setupPath = (string)file.Properties["vti_setuppath"];
                    string rootPath = SPUtility.GetGenericSetupPath("Template");

                    if (!File.Exists(Path.Combine(rootPath, setupPath)))
                    {
                        output += string.Format("The template file ({0}) does not exist so re-ghosting will not be possible.", Path.Combine(rootPath, setupPath));

                        // something's wrong with the setup path - lets see if we can fix it
                        // Try and remove a leading locale if present
                        setupPath = "SiteTemplates\\" + setupPath.Substring(5);
                        if (File.Exists(Path.Combine(rootPath, setupPath)))
                        {
                            output += string.Format("  It appears that a possible template match does exist at \"{0}\" however this tool currently is not able to handle pointing the file to the correct template path. This scenario is most likely due to an upgrade from SPS 2003.", Path.Combine(rootPath, setupPath));

                            // We found a matching file so reset the property and update the file.
                            // ---  I wish this would work but it simply doesn't - something is preventing the
                            //      update from occuring.  Manipulating the database directly results in a 404
                            //      when attempting to load the "fixed" page so there's gotta be something beyond
                            //      just updating the setuppath property.
                            //file.Properties["vti_setuppath"] = setupPath;
                            //file.Update();
                        }
                    }
                }
            }
            file.RevertContentStream();

            #region
            //file = web.GetFile(url);
            //if (file.CustomizedPageStatus == SPCustomizedPageStatus.Customized)
            //{
            //    // Still unsuccessful so take measures further
            //    if (force)
            //    {
            //        PropertyInfo requestProp = web.GetType().GetProperty("Request",
            //                                                             BindingFlags.NonPublic |
            //                                                             BindingFlags.Instance |
            //                                                             BindingFlags.InvokeMethod |
            //                                                             BindingFlags.GetProperty);
            //        object request = requestProp.GetValue(web, null);

            //        MethodInfo revertContentStreams =
            //            request.GetType().GetMethod("RevertContentStreams",
            //                                        BindingFlags.NonPublic | BindingFlags.Public |
            //                                        BindingFlags.Instance | BindingFlags.InvokeMethod);

            //        // I found some cases where calling this directly was the only way to force the re-ghosting of the file.
            //        // I think the trick is that it's not updating the file properties after doing the revert (the
            //        // RevertContentStream method will call SPRequest.UpdateFileOrFolderProperties() immediately after the 
            //        // RevertContentStreams call but ommitting the update call seems to make a difference.
            //        revertContentStreams.Invoke(request,
            //                                    new object[]
            //                                        {
            //                                            web.Url, file.Url,
            //                                            file.CheckOutStatus != SPFile.SPCheckOutStatus.None
            //                                        });
            //        MethodInfo dirtyThisFileObject =
            //            file.GetType().GetMethod("DirtyThisFileObject",
            //                                     BindingFlags.NonPublic | BindingFlags.Public |
            //                                     BindingFlags.Instance | BindingFlags.InvokeMethod);
            //        dirtyThisFileObject.Invoke(file, new object[] { });

            //        file = web.GetFile(url);

            //        if (file.CustomizedPageStatus == SPCustomizedPageStatus.Customized)
            //            output += "Unable to re-ghost file " + file.ServerRelativeUrl;
            //        else
            //            output += file.ServerRelativeUrl + " was re-ghosted!";

            //    }
            //    else
            //        output += "Unable to re-ghost file " + file.ServerRelativeUrl;
            //}
            //else
            #endregion

            output += file.ServerRelativeUrl + " was re-ghosted!";
            return output;
        }
    }
}
